#include <stdio.h>
#include <conio.h>

int main(int argc, char **argv)
{
   printf("\n");
   printf("Tektronix 49x ROM checksum utility V1.00 of 20-May-04\n");
   printf("John Miles KE5FX (jmiles@pop.net)\n");
   printf("-------------------------------------------------------------------------------\n");

   if ((argc < 3) 
         || 
      ((argv[2][0] != '0') && (argv[2][0] != '1')))
      {
      printf("\nUsage: chksum filename.bin bank\n");
      printf("Bank = 0 for first 16K bank in image, 1 for second 16K bank\n\n");
      return 1;
      }

   static unsigned char image[32768];

   FILE *in = fopen(argv[1],"rb");

   if (in == NULL)
      {
      printf("Error: could not find %s\n",argv[1]);
      return 1;
      }

   if (fread(image, 32768, 1, in) != 1)
      {
      printf("Error: could not read %s\n",argv[1]);
      return 1;
      }

   fclose(in);

   int start = 0;
   int end   = 16383;

   if (argv[2][0] == '1')
      {
      start = 16384;
      end   = 32767;
      }

   unsigned short original = *(unsigned short *) &image[start];

   printf("Previous checksum: %X\n",original);

   //
   // Original 6802 checksum algorithm:
   //
   //          CLR A               ;Init sum = 0 (A=MSB, B=LSB)
   //          CLR B
   //
   // L_DBC4   ASL B               ;16-bit sum <<= 1
   //          ROL A               ;Shift MSB out to carry
   //          ADC B   $00,X       ;16-bit add of $00,x plus carry -> sum
   //          ADC A   #$00
   //          INX                 ;Loop from $C002 to FFFF inclusive
   //          CPX     #$0000
   //          BNE     L_DBC4   
   //

   unsigned short sum = 0;

   for (int i=start+2; i <= end; i++)   
      {
      unsigned short C = (sum & 0x8000) ? 1 : 0;
      sum <<= 1;

      sum = sum + C + image[i];
      }

   image[start  ] = (sum >> 8) & 0xff;
   image[start+1] =  sum       & 0xff;

   unsigned short updated = *(unsigned short *) &image[start];

   printf("     New checksum: %X\n\n",updated);

   if (updated == original)
      {
      printf("     Checksum is unchanged\n");
      return 0;
      }

   fflush(stdin);
   printf("Press 'y' to update %s, any other key to exit...\n",argv[1]);
   
   char ch = getch();
   if ((ch == 'y') || (ch == 'Y'))
      {
      FILE *out = fopen(argv[1], "wb");

      if (out == NULL)
         {
         printf("Error: could not open %s for writing\n",argv[1]);
         return 1;
         }

      if (fwrite(image, 32768, 1, in) != 1)
         {
         printf("Error: could not write to %s\n",argv[1]);
         return 1;
         }

      fclose(out);
      printf("OK\n");
      }

   return 0;
}
